@@ -20,7 +20,7 @@
 """
 from .deprecated import _custom_ioa_deprecated
 from .deprecated import _firewall_management_deprecated
-# from .deprecated import _identity_detections_deprecated
+from .deprecated import _identity_protection_deprecated
 from .deprecated import _installation_tokens_deprecated
 from .deprecated import _ioc_deprecated
 from .deprecated import _iocs_deprecated
@@ -40,7 +40,7 @@ from ._firewall_management import _firewall_management_endpoints
 from ._firewall_policies import _firewall_policies_endpoints
 from ._host_group import _host_group_endpoints
 from ._hosts import _hosts_endpoints
-# from ._identity_detections import _identity_detections_endpoints
+from ._identity_protection import _identity_protection_endpoints
 from ._incidents import _incidents_endpoints
 from ._installation_tokens import _installation_tokens_endpoints
 from ._intel import _intel_endpoints
@@ -81,7 +81,7 @@ api_endpoints.extend(_firewall_management_endpoints)
 api_endpoints.extend(_firewall_policies_endpoints)
 api_endpoints.extend(_host_group_endpoints)
 api_endpoints.extend(_hosts_endpoints)
-# api_endpoints.extend(_identity_detections_endpoints)
+api_endpoints.extend(_identity_protection_endpoints)
 api_endpoints.extend(_incidents_endpoints)
 api_endpoints.extend(_installation_tokens_endpoints)
 api_endpoints.extend(_intel_endpoints)
@@ -112,7 +112,7 @@ api_endpoints.extend(_zero_trust_assessment_endpoints)
 deprecated_endpoints = []
 deprecated_endpoints.extend(_custom_ioa_deprecated)
 deprecated_endpoints.extend(_firewall_management_deprecated)
-# deprecated_endpoints.extend(_identity_detections_deprecated)
+deprecated_endpoints.extend(_identity_protection_deprecated)
 deprecated_endpoints.extend(_installation_tokens_deprecated)
 deprecated_endpoints.extend(_ioc_deprecated)
 deprecated_endpoints.extend(_iocs_deprecated)
